/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------
   
   Functions related to asking for a password when none defined.
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <string.h>
#include <stdio.h>

#include "ask.h"
#include "choose.h"
#include "confread.h"
#include "read.h"
#include "xmalloc.h"
#include "cmd.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static char *
make_prompt (char *p)
{
  int   len    = strlen (p);
  char *result = xmalloc (len + 3);

  sprintf (result, "%s: ", p);
  return result;
}


static void
hide_passwd (rstring_t *items)
{
  int   i;
  char *seek;

  for (i = 0; i < items->count; i++){
    seek = strchr (items->array[i], ':');
    if (seek)
      *seek = '\0';
  }
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

char *
ask_value (char *name)
{
  int        index;
  char      *prompt;
  char      *txt;
  rstring_t *list;

  txt  = cf_txtvar_lookup (name);
  if (txt)
    return txt;
  list = cf_listvar_lookup (name);
  if (list == NULL){
    prompt = make_prompt (name);
    txt    = read_argument (prompt, NULL);
    xfree (prompt);
    return txt;
  }

  if (list->count == 1){
    return list->array[0];
  }

  choose_open (list->count, list->array);
  cmd_choose ();

  index = choose_index ();

  if (index == -1)
    return NULL;
  
  return list->array[index];
}



char *
ask_noninteractive (char *name)
{
  char      *txt;
  rstring_t *list;

  txt = cf_txtvar_lookup (name);
  if (txt)
    return txt;
  list = cf_listvar_lookup (name);
  if (list == NULL || list->count != 1)
    return NULL;
  return list->array[0];
}



char *
ask_hide_passwd (char *name)
{
  int        index;
  char      *txt;
  char      *prompt;
  rstring_t *list;
  rstring_t *disp_list;

  txt = cf_txtvar_lookup (name);
  if (txt)
    return txt;
  list = cf_listvar_lookup (name);
  if (list == NULL){
    prompt = make_prompt (name);
    txt    = read_argument (prompt, NULL);
    xfree (prompt);
    return txt;
  }

  if (list->count == 1){
    return list->array[0];
  }

  disp_list = rstring_copy (list);
  hide_passwd (disp_list);

  choose_open (disp_list->count, disp_list->array);
  cmd_choose ();

  rstring_free_strings (disp_list);
  rstring_delete (disp_list);
  
  index = choose_index ();

  if (index == -1)
    return NULL;
  
  return list->array[index];
}




int
ask_if_sure (char *question)
{
  char *answer = read_argument (question, NULL);

  if (*answer == 'y' || *answer == 'Y')
    return 1;
  return 0;
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE ask.c
 *
 ****************************************************************************/
