/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002, 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   This is an example file to examine libraries and to play some with
   them.
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#define _GNU_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <regex.h>
#include <locale.h>

#include "token.h"
#include "xmalloc.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static char long_txt[] = "Baka miaa fajny biust\n"
"Ala styl, a Goka co co lubi.\n"
"Tylko u nas za jedyne $100 lij na ala@kot.org. 12 32";

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    FUNCTION SUBSTITUTES
 ****************************************************************************/

char *
prepare_error_msg (int errnum, const char *fmt, va_list ap)
{
  char *result;
  char *user_msg;
  int   user_len;
  char *system_msg;
  int   system_len;
  
  user_len = vasprintf (&user_msg, fmt, ap);

  if (errnum){
    system_msg = strerror (errnum);
    system_len = strlen (system_msg);
    result     = xmalloc (user_len + 2 + system_len + 1);
    memcpy (result, user_msg, user_len);
    memcpy (result + user_len + 2, system_msg, system_len + 1);
    result[user_len]     = ':';
    result[user_len + 1] = ' ';
  }
  else {
    result = xstrdup (user_msg);
  }

  free (user_msg);
  return result;
}



void
error_ (int status, int errnum , const char *fmt, ...)
{
  va_list  ap;
  char    *msg;

  va_start (ap, fmt);
  msg = prepare_error_msg (errnum, fmt, ap);
  va_end (ap);

  fprintf (stderr, "%s\n", msg);

  xfree (msg);

  if (status){
    exit (status);
  }
}


void
error_regex (int errcode, regex_t *compiled)
{
  int   length = regerror (errcode, compiled, NULL, 0);
  char *buffer = xmalloc (length + 1);

  regerror (errcode, compiled, buffer, length);

  fprintf (stderr, "%s\n", buffer);
  xfree (buffer);
}

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static int
token_test (void)
{
  int   td;
  char *word;

  token_init ();

  td = token_open (long_txt, 5, TOKEN_BAYES_WORD_RE, 1);

  if (td == -1)
    return 1;

  while (1){
    word = token_read_next (td);
    if (word == NULL)
      break;
    printf ("%s\n", word);
  }
  token_close (td);
  return 0;
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


void
usage (int status)
{
  fprintf (stderr, "exampl [x]\n"
           "  0 - this help\n"
           "  1 - token test\n");
  exit (status);
}


int
main (int argc, char **argv){
  
  setlocale (LC_ALL, "");

  if (argc > 1){
    switch (*argv[1]){

      case '0':
        usage (EXIT_SUCCESS);
        break;

      case '1':
        return token_test ();
    
      default:
        usage (EXIT_FAILURE);
        break;
    }
  }

  exit (EXIT_SUCCESS);
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE exampl.c
 *
 ****************************************************************************/
