/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   mail fetching window
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>

#include "ecurses.h"
#include "pop.h"
#include "error.h"
#include "select.h"
#include "line.h"
#include "xmalloc.h"
#include "maildir.h"
#include "misc.h"
#include "cmd.h"
#include "file.h"
#include "ask.h"
#include "lcs.h"
#include "fetch.h"
#include "gettext.h"
#include "mybox.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define WINDOW_WIDTH COLS - 6

#define PREAMBLE do { if (fetch_select == NULL) return; } while (0)

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static select_t *fetch_select = NULL;

static char *host = NULL;


static int         prev_stat_size = 0;
static pop_stat_t *prev_stat_list = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/

int fetch_got_mail = 0;

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/


static rstring_t *
make_items (void)
{
  int        i;
  line_t    *line   = line_create (WINDOW_WIDTH - 2);
  rstring_t *result = rstring_create ();

  for (i = 0; i < pop_stat_size; i++){
    if (pop_stat_list[i].mark == 1)
      continue;
    line_clear (line);
    line_add_char (line, ' ');
    line_add_int (line, pop_stat_list[i].num);
    line_add_char (line, ' ');
    line_add_str (line, human_size_paren (pop_stat_list[i].size));
    line_add_char (line, ' ');
    if (pop_stat_list[i].info){
      if (pop_stat_list[i].info->from)
        line_add_str_pad (line, pop_stat_list[i].info->from->name_short,
                          ADDR_NAME_LEN);
      line_add_char (line, ' ');
      if (pop_stat_list[i].info->subject)
        line_add_str_dots (line, pop_stat_list[i].info->subject);
    }
    rstring_add (result, xstrdup (line->str));
  }
  line_destroy (line);
  return result;
}



static int
stats_equal (void *a, void *b)
{
  pop_stat_t *aa = (pop_stat_t *) a;
  pop_stat_t *bb = (pop_stat_t *) b;

  return aa->size == bb->size;
}



static void
mark_stat (void *a, void *b)
{
  pop_stat_t *aa = (pop_stat_t *) a;
  pop_stat_t *bb = (pop_stat_t *) b;

  if (bb->mark == 1)
    aa->mark = 1;
}



static int
selected_index (void)
{
  int i;
  int index = 0;

  for (i = 0; i < pop_stat_size; i++){
    if (pop_stat_list[i].mark == 0 || pop_stat_list[i].fetched)
      index++;
    if (index > fetch_select->bar_pos)
      break;
  }
  if (i < pop_stat_size)
    return i;
  return -1;
}


static int
selected_num (void)
{
  int index = selected_index ();

  if (index == -1)
    return -1;
  return pop_stat_list[index].num;
}



static int
begin_connection (void)
{
  rstring_t *conf;
  char      *ask;
  char      *copy;
  int        port = 0;
  char      *user;
  char      *pass;

  ask = ask_hide_passwd ("pop_acc");

  if (ask == NULL)
    return 1;

  copy = xstrdup (ask);
  conf = rstring_split (copy, ":");

  if (conf->count < 3){
    error_ (0, 0, "invalid pop_acc value '%s'", ask);
    return 1;
  }
  
  host = xstrdup (conf->array[0]);
  user = conf->array[1];
  pass = conf->array[2];

  if (conf->count > 3){
    port = atoi (conf->array[3]);
  }

  fetch_got_mail = 0;

  if (pop_begin_connection (host, (unsigned short) port)){
    rstring_delete (conf);
    xfree (copy);
    return 1;
  }

  if (pop_authorize (user, pass)){
    rstring_delete (conf);
    xfree (copy);
    return 1;
  }
  
  rstring_delete (conf);
  xfree (copy);
  
  if (pop_list ())
    return 1;

  return 0;
}



static int
get_mail (int index)
{
  int     fd;
  char   *fname;
  char   *dir;
  FILE   *fp;

  dir = mybox_incoming ();
  if (dir == NULL){
    error_ (0, 0, _("no place to write... fetching aborted"));
    return 1;
  }
  
  fname = maildir_tmp_file (dir);
  fd    = open (fname, O_RDWR | O_CREAT | O_EXCL, 0600);
  if (fd == -1){
    error_ (0, errno, _("couldn't write in directory %s"), dir);
    return 1;
  }

  fp = fdopen (fd, "w");
  if (!fp){
    error_ (0, errno, "%s", fname);
    return 1;
  }

  if (pop_retr (index, fp)){
    error_ (0, 0, _("couldn't fetch mail %d"), index);
    fclose (fp);
    xfree (fname);
    return 1;
  }

  fclose (fp);

  maildir_move_to_new (fname);
  xfree (fname);
  xfree (dir);

  return 0;
}



static void
no_new_mail (rstring_t *items)
{
  pop_finish_connection ();
  rstring_free_strings (items);
  rstring_delete (items);
  xfree (host);
  host = NULL;
  error_ (0, 0, _("no new mail"));
  cmd_refresh ();
}



static void
open_window (void)
{
  rstring_t *items;

  if (pop_list_ext ())
    return;

  items = make_items ();
  if (items->count == 0){
    fetch_save_pop_list ();
    no_new_mail (items);
    return;
  }

  fetch_select = select_open (items, WINDOW_WIDTH, 20, 1, 0, _("fetch"));

  if (fetch_select){
    select_show (fetch_select);
    cmd_state_push (CMD_FETCH);
  }
}



/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


void
fetch_all (void)
{
  int i;
  
  if (begin_connection ())
    return;

  fetch_load_pop_list ();
  fetch_merge_lists ();
  
  for (i = 0; i < pop_stat_size; i++){
    if (pop_stat_list[i].mark == 1)
      continue;
    if (get_mail (pop_stat_list[i].num) == 0)
      pop_stat_list[i].mark = 1;
  }

  fetch_save_pop_list ();

  if (host){
    xfree (host);
    host = NULL;
  }
  
  pop_finish_connection ();
}




void
fetch_get_rest (void)
{
  int        i;
  rstring_t *items;
  
  PREAMBLE;

  for (i = 0; i < pop_stat_size; i++){
    pop_stat_list[i].mark = 0;
  }

  if (pop_list_ext ())
    return;

  items = make_items ();
  if (items->count == 0){
    no_new_mail (items);
    fetch_close ();
    return;
  }

  select_install_new_items (fetch_select, items);
}



void
fetch_open (void)
{
  if (begin_connection ())
    return;
  
  fetch_load_pop_list ();
  fetch_merge_lists ();
  
  open_window ();
}



void
fetch_open_all (void)
{
  if (begin_connection ())
    return;
  
  open_window ();
}



void
fetch_close (void)
{
  PREAMBLE;

  fetch_save_pop_list ();

  if (host){
    xfree (host);
    host = NULL;
  }
  
  pop_finish_connection ();
  rstring_free_strings (fetch_select->items);
  rstring_delete (fetch_select->items);
  select_close (fetch_select);
  fetch_select = NULL;

  cmd_state_pop ();
}



void
fetch_next (void)
{
  PREAMBLE;

  select_next (fetch_select);
}



void
fetch_prev (void)
{
  PREAMBLE;

  select_prev (fetch_select);
}



void
fetch_next_page (void)
{
  PREAMBLE;

  select_next_page (fetch_select);
}



void
fetch_prev_page (void)
{
  PREAMBLE;

  select_prev_page (fetch_select);
}



void
fetch_first (void)
{
  PREAMBLE;

  select_first (fetch_select);
}



void
fetch_last (void)
{
  PREAMBLE;

  select_last (fetch_select);
}



void
fetch_rset (void)
{
  int i;
  
  PREAMBLE;

  pop_rset ();
  for (i = 0; i < fetch_select->items->count; i++){
    *fetch_select->items->array[i] = ' ';
  }
  select_redraw (fetch_select);
}



void
fetch_del_all (void)
{
  int i;
  
  if (begin_connection ())
    return;

  for (i = 0; i < pop_stat_size; i++){
    pop_dele (i);
  }

  if (host){
    xfree (host);
    host = NULL;
  }
  
  pop_finish_connection ();
}



void
fetch_del_mail (void)
{
  int index = selected_num ();

  PREAMBLE;

  pop_dele (index);
  *fetch_select->items->array[fetch_select->bar_pos] = 'D';
  select_redraw (fetch_select);
}



void
fetch_get_mail (void)
{
  int index;
  
  PREAMBLE;

  index = selected_num ();

  if (get_mail (index) == 0){
    *fetch_select->items->array[fetch_select->bar_pos] = 'F';
    select_redraw (fetch_select);
    fetch_got_mail = 1;
    fetch_mark_mail ();
  }
}


void
fetch_mark_mail (void)
{
  int index;
  
  PREAMBLE;

  index = selected_index ();
  pop_stat_list[index].fetched = 1;
}



void
fetch_unmark_mail (void)
{
  int index;
  
  PREAMBLE;

  index = selected_index ();
  pop_stat_list[index].fetched = 0;
}



void
fetch_mark_all (void)
{
  int i;

  PREAMBLE;

  for (i = 0; i < pop_stat_size; i++){
    pop_stat_list[i].fetched = 1;
  }
}



void
fetch_unmark_all (void)
{
  int i;

  PREAMBLE;

  for (i = 0; i < pop_stat_size; i++){
    pop_stat_list[i].fetched = 0;
  }
}



void
fetch_save_pop_list (void)
{
  char *dir;
  char *fname;
  FILE *fp;
  int   i;
  
  dir = ask_value ("pop_info_dir");
  if (dir == NULL){
    error_ (0, 0, "no pop3 info dir");
    return;
  }

  fname = file_with_dir (dir, host);
  fp    = fopen (fname, "w");

  if (fp == NULL){
    error_ (0, errno, "%s", fname);
    return;
  }

  fprintf (fp, "%d\n", pop_stat_size);
  for (i = 0; i < pop_stat_size; i++){
    fprintf (fp, "%d %d %d\n", pop_stat_list[i].num, pop_stat_list[i].size,
             pop_stat_list[i].mark | pop_stat_list[i].fetched);
  }

  fclose (fp);
  xfree (fname);
}



void
fetch_load_pop_list (void)
{
  char *dir;
  char *fname;
  FILE *fp;
  int   i;

  dir = ask_value ("pop_info_dir");
  if (dir == NULL){
    error_ (0, 0, "no pop3 info dir");
    return;
  }

  fname = file_with_dir (dir, host);
  fp    = fopen (fname, "r");

  if (fp == NULL){
    xfree (fname);
    return;
  }

  fscanf (fp, "%d\n", &prev_stat_size);

  prev_stat_list = xmalloc (prev_stat_size * sizeof (pop_stat_t));
  for (i = 0; i < prev_stat_size; i++){
    fscanf (fp, "%d %d %d\n", &prev_stat_list[i].num, &prev_stat_list[i].size,
            &prev_stat_list[i].mark);
  }

  fclose (fp);
  xfree (fname);
}



void
fetch_merge_lists (void)
{
  lcs_t *lcs;

  if (prev_stat_list == NULL)
    return;
  
  lcs = lcs_compute (pop_stat_list,       prev_stat_list,
                     sizeof (pop_stat_t), sizeof (pop_stat_t),
                     pop_stat_size,       prev_stat_size, stats_equal);
  lcs_mark_part (lcs, pop_stat_list,      prev_stat_list,
                 sizeof (pop_stat_t),     sizeof (pop_stat_t), mark_stat);
  lcs_destroy (lcs);

  if (prev_stat_list){
    xfree (prev_stat_list);
    prev_stat_list = NULL;
    prev_stat_size = 0;
  }
}



/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE fetch.c
 *
 ****************************************************************************/
