/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   help window
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include "select.h"
#include "help.h"
#include "cmd.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define HELP_WIDTH  80
#define HELP_HEIGHT 20

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static select_t *help_win = NULL;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static rstring_t *
make_items (void)
{
  rstring_t *result = rstring_create ();

  rstring_add (result, _("Welcome to elmo help pages."));
  rstring_add (result, _("Current key combinations are:"));
  
  return result;
}



static void
destroy_items (rstring_t *items)
{
  rstring_delete (items);
}

/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
help_open (void)
{
  rstring_t *items = make_items ();

  help_win = select_open (items, HELP_WIDTH, HELP_HEIGHT, 1, 1, _("help"));

  if (help_win == NULL){
    destroy_items (items);
    return;
  }

  select_show (help_win);
  cmd_state_push (CMD_HELP);
}




void
help_close (void)
{
  destroy_items (help_win->items);
  select_close (help_win);
  cmd_state_pop ();
}



void
help_scroll_down (void)
{
  select_next (help_win);
}



void
help_scroll_up (void)
{
  select_prev (help_win);
}



void
help_prev_page (void)
{
  select_prev_page (help_win);
}



void
help_next_page (void)
{
  select_next_page (help_win);
}



void
help_beg (void)
{
  select_first (help_win);
}



void
help_end (void)
{
  select_last (help_win);
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE help.c
 *
 ****************************************************************************/
