/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 bazyl
   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   showing e-mail in scrolling window
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <unistd.h>
#include <errno.h>

#include "ecurses.h"
#include "mailreader.h"
#include "screen.h"
#include "rchtype.h"
#include "status.h"
#include "error.h"
#include "xmalloc.h"
#include "attach.h"
#include "cmd.h"
#include "wrapbox.h"
#include "folder.h"
#include "gettext.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static int        curr_line = 0;
static WINDOW    *mail_win;
static rchtype_t *mail_lines;

static char *mail_body;
static int   mail_index;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/

static void       write_lines (void);
static rchtype_t *generate_header (int index);

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static void
write_lines (void)
{
  int i;
  
  if (curr_line < 0)
    curr_line = 0;
  if (curr_line >= mail_lines->count)
    curr_line = mail_lines->count - 2;
  
  wclear (mail_win);

  for (i = 0; i < LINES - 2 && i + curr_line < mail_lines->count; i++){
    mvwaddchstr (mail_win, i, 0, mail_lines->array[i + curr_line]);
  }
}



static rchtype_t *
generate_header (int index)
{
  rchtype_t *result = rchtype_create ();
  int        i;

  rchtype_set_mask (COLOR_PAIR (5));
  if (mail_array[index].date_str)
    rchtype_add_string (result, "Date: %s", mail_array[index].date_str);
  if (mail_array[index].from)
    rchtype_add_string (result, "From: %s", mail_array[index].from->full);
  if (mail_array[index].subject)
    rchtype_add_string (result, "Subject: %s", mail_array[index].subject);
  if (mail_array[index].to && mail_array[index].to->count){
    rchtype_add_string (result, "To: %s",
                        mail_array[index].to->array[0]->full);
    for (i = 1; i < mail_array[index].to->count; i++){
      rchtype_add_string (result, "    %s",
                          mail_array[index].to->array[i]->full);
    }
  }
  if (mail_array[index].cc && mail_array[index].cc->count){
    rchtype_add_string (result, "Cc: %s",
                        mail_array[index].cc->array[0]->full);
    for (i = 1; i < mail_array[index].cc->count; i++){
      rchtype_add_string (result, "    %s",
                          mail_array[index].cc->array[i]->full);
    }
  }
  if (mail_array[index].bcc && mail_array[index].bcc->count){
    rchtype_add_string (result, "To: %s",
                        mail_array[index].bcc->array[0]->full);
    for (i = 1; i < mail_array[index].bcc->count; i++){
      rchtype_add_string (result, "    %s",
                          mail_array[index].bcc->array[i]->full);
    }
  }
  if (mail_array[index].reply_to
      && mail_array[index].reply_to != mail_array[index].from)
    rchtype_add_string (result, "Reply-To: %s",
                        mail_array[index].reply_to->full);
  if (mail_array[index].mua != MUA_UNKNOWN){
    if (MUA_CONSOLE (mail_array[index].mua)){
      rchtype_set_mask (A_BOLD);
    }
    rchtype_add_string (result, "MUA: %s",
                        mail_mua_names[mail_array[index].mua]);
    if (MUA_CONSOLE (mail_array[index].mua)){
      rchtype_unset_mask (A_BOLD);
    }
  }
  rchtype_unset_mask (COLOR_PAIR (5));
  rchtype_add_string (result, "");
  return result;
}



static int
set_mail_lines (int mime_index)
{
  mail_t    *mail   = mail_array + mail_index;
  rchtype_t *header = generate_header (mail_index);
  
  if (wrapbox_mail_body (mail, &mail_body, mime_index) && errno){
    error_ (0, errno, _("impossible to display mail contents"));
    return 1;
  }

  if (mail_lines){
    rchtype_destroy (mail_lines);
    mail_lines = NULL;
  }
  
  if (mail_body){
    mail_lines = rchtype_split_lines (mail_body, COLS);
    mail_lines = rchtype_join (header, mail_lines);
  }
  else {
    mail_lines = header;
  }

  if (mail_body){
    xfree (mail_body);
    mail_body = NULL;
  }
  return 0;
}



/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
mailreader_line_up (void)
{
  if (curr_line == 0)
    return;
  curr_line--;
  wscrl (mail_win, -1);
  mvwaddchstr (mail_win, 0, 0, mail_lines->array[curr_line]);
  screen_wnoutrefresh (mail_win);
}



void
mailreader_line_down (void)
{
  if (curr_line >= (int)(mail_lines->count - LINES + 2))
    return;
  curr_line++;
  wscrl (mail_win, 1);
  mvwaddchstr (mail_win, LINES - 3, 0,
               mail_lines->array[curr_line + LINES - 3]);
  screen_wnoutrefresh (mail_win);
}



void
mailreader_page_up (void)
{
  curr_line -= LINES - 2;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_page_down (void)
{
  curr_line += LINES - 2;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_top (void)
{
  curr_line = 0;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_bottom (void)
{
  curr_line = mail_lines->count - LINES + 2;
  write_lines ();
  screen_wnoutrefresh (mail_win);
}



void
mailreader_change_mime (void)
{
  int     mime_index = attach_selected_item;
  mime_t *mime;

  if (mime_index == -1)
    return;

  mime = mail_array[mail_index].rmime->array[mime_index];

  if (mime->type == MTYPE_TEXT_PLAIN ||
      mime->type == MTYPE_TEXT_OTHER ||
      mime->type == MTYPE_HTML       ||
      mime->type == MTYPE_MESSAGE){
  
    set_mail_lines (mime_index);

    curr_line  = 0;
    write_lines ();
  }
}



void
mailreader_init (void)
{
  int mail_num = folder_bar_index ();
  
  mail_index = mail_num;
  mail_win   = newwin (LINES - 2, COLS, 0, 0);

  scrollok (mail_win, TRUE);
  
  if (set_mail_lines (-1))
    return;

  curr_line  = 0;
  write_lines ();

  status_put_mailinfo (mail_array[mail_num].from->name,
                       mail_array[mail_num].subject);
  
  screen_touchwin (mail_win);
  screen_wnoutrefresh (mail_win);
  screen_doupdate ();

  cmd_state_push (CMD_READ_MAIL);
}



void
mailreader_close (void)
{
  delwin(mail_win);
  rchtype_destroy (mail_lines);
  mail_lines = NULL;

  cmd_state_pop ();
}



void
mailreader_attach (void)
{
  attach_open (mail_array + mail_index);
}



void
mailreader_refresh (void)
{
  int mail_num = folder_bar_index ();
  
  screen_touchwin (mail_win);
  screen_wnoutrefresh (mail_win);

  status_put_mailinfo (mail_array[mail_num].from->name,
                       mail_array[mail_num].subject);
  status_refresh ();
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mailreader.c
 *
 ****************************************************************************/
