/* 
   elmo - Electronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------


   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "mime.h"
#include "xmalloc.h"
#include "file.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

#define BASE64_VAL(a) (isupper (a) ? ((a) - 'A') \
                       : (islower (a) ? ((a) - 'a' + 26) \
                          : (isdigit (a) ? ((a) - '0' + 52) \
                             : ((a) == '+' ? 62 \
                                : ((a) == '/' ? 63 : 0)))))

#define BASE64_SIGN(a) (((a) < 26) ? ('A' + (a)) \
                        : (((a) < 52) ? ('a' + (a) - 26) \
                           : (((a) < 62) ? ('0' + (a) - 52) \
                              : (((a) == 62) ? '+' \
                                 : (((a) == 63) ? '/' : 0)))))
                        

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/

char *mime_enc_str[] = {
  "8bit",
  "quoted-printable",
  "base64",
};

char *mime_type_str[] = {
  "application/octet-stream",
  "text/plain; charset=iso-8859-2",
  "text/html",
  "message/rfc822",
  "application/octet-stream",
  "application/octet-stream",
  "application/octet-stream",
};

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

/*
 * Ala ma kota =?iso-8859-2?Q?kt=ACry_jest_chory?= he, he.
 *             ^              ^                ^
 *             |              |                |
 *             +---- start    +---- t          +---- end
 */
static mime_enc_t
find_mime_in_string (char *s, char **t, char **start, char **end)
{
  mime_enc_t result;
  
  *start = strstr (s, "=?");
  if (*start == NULL){
    return MENC_NONE;
  }
  *t = *start + 2;

  while (**t && **t != '?')
    (*t)++;
  (*t)++;

  switch (**t){

    case 'q': case 'Q':
      result = MENC_QP;
      break;

    case 'b': case 'B':
      result = MENC_BASE64;
      break;

    default:
      result = MENC_NONE;
  }
  *t += 2;

  *end = strstr (*t, "?=");
  if (*end == NULL){
    *t = NULL;
    return MENC_NONE;
  }
  (*end)--;
  return result;
}




/**
 * to -- inclusive
 */
static char *
decode_qp (char *dest, char *from, char *to, int special_)
{
  char buf[3];

  buf[2] = '\0';
  
  while (*from && from <= to){

    switch (*from){

      case '_':
        if (special_){
          *dest = ' ';
        }
        else {
          *dest = *from;
        }
        dest++;
        from++;
        break;
      
      case '=':
        if (isxdigit (from[1]) && isxdigit (from[2])){
          buf[0] = from[1];
          buf[1] = from[2];
          *dest  = (unsigned char) strtol (buf, NULL, 16);
          dest += 1;
          from += 3;
        }
        else if (from[1] == '\n'){
          from += 2;
        }
        else {
          from += 3;
        }
        break;

      default:
        *dest = *from;
        dest++;
        from++;
        break;
    }
  }
  return dest;
}



static char *
decode_base64 (char *dest, char *from, char *to)
{
  while (*from && from <= to){
    while (*from == '\n' || *from == '\r')
      from++;
    *dest = (BASE64_VAL (from[0]) << 2) | (BASE64_VAL (from[1]) >> 4);
    dest++;

    if (from[2] == '=')
      break;
    *dest = (BASE64_VAL (from[1]) << 4) | (BASE64_VAL (from[2]) >> 2);
    dest++;

    if (from[3] == '=')
      break;
    *dest = (BASE64_VAL (from[2]) << 6) | (BASE64_VAL (from[3]) >> 0);
    dest++;
    from += 4;
  }
  return dest;
}



static char *
encode_base64 (unsigned char *dest, unsigned char *from, unsigned char *to)
{
  int line_len = 0;

  while (from <= to){

    *dest = BASE64_SIGN   (0x3f & (                  from[0] >> 2));
    dest++;

    if (from + 1 > to){
      *dest = BASE64_SIGN (0x3f & (from[0] << 4));
      dest++;
      *dest = '=';
      dest++;
      *dest = '=';
      dest++;
      break;
    }
    *dest = BASE64_SIGN   (0x3f & ((from[0] << 4) | (from[1] >> 4)));
    dest++;

    if (from + 2 > to){
      *dest = BASE64_SIGN (0x3f &  (from[1] << 2));
      dest++;
      *dest = '=';
      dest++;
      break;
    }
    *dest = BASE64_SIGN   (0x3f & ((from[1] << 2) | (from[2] >> 6)));
    dest++;

    *dest = BASE64_SIGN   (0x3f &  (from[2]));
    dest++;

    if (line_len == 76){
      *dest    = '\n';
      dest++;
      line_len = 0;
    }
    else {
      line_len += 4;
    }
    from += 3;
  }
  *dest = '\n';
  dest++;
  return dest;
}



static mime_type_t
get_type (char *header)
{
  char *seek;
  
  while (!isalpha (*header))
    header++;

  switch (*header){

    case 'x': case 'X':
      return MTYPE_X_SOMETHING;

    case 'i': case 'I':
      if (strstr (header + 1, "mage") == header + 1)
        return MTYPE_IMAGE;
      if (strstr (header + 1, "MAGE") == header + 1)
        return MTYPE_IMAGE;
      break;

    case 'a': case 'A':
      if (strstr (header + 1, "pplication") == header + 1)
        return MTYPE_APPLICATION;
      if (strstr (header + 1, "PPLICATION") == header + 1)
        return MTYPE_APPLICATION;
      break;
    
    case 'm': case 'M':
      if (strstr (header + 1, "essage") == header + 1)
        return MTYPE_MESSAGE;
      if (strstr (header + 1, "ESSAGE") == header + 1)
        return MTYPE_MESSAGE;
      break;
    
    case 't': case 'T':
      seek = strstr (header + 1, "ext");
      if (seek == NULL)
        seek = strstr (header + 1, "EXT");
      if (seek == NULL)
        break;
      if (seek[3] != '/')
        return MTYPE_TEXT_OTHER;
      if (strstr (seek + 4, "plain") == seek + 4)
        return MTYPE_TEXT_PLAIN;
      if (strstr (seek + 4, "html") == seek + 4)
        return MTYPE_HTML;
      if (strstr (seek + 4, "PLAIN") == seek + 4)
        return MTYPE_TEXT_PLAIN;
      if (strstr (seek + 4, "HTML") == seek + 4)
        return MTYPE_HTML;
      return MTYPE_TEXT_OTHER;
      break;

    default:
      return MTYPE_UNKNOWN;
  }

  return MTYPE_UNKNOWN;
}



static char *
get_file_name (char *header)
{
  char *seek = strstr (header, "name=");
  char *start;
  char *result;

  if (seek == NULL)
    seek = strstr (header, "NAME=");
  
  if (seek == NULL)
    return NULL;

  start = seek + 5;
  if (*start == '"')
    start++;

  seek = start;
  while (*seek != '"' && *seek != '\r' && *seek != '\n' && *seek){
    seek++;
  }
  result = xmalloc (seek - start + 1);
  result[seek - start] = '\0';
  memcpy (result, start, seek - start);
  return mime_decode_header (result, 1);
}



static char *
get_desc (char *header)
{
  char *start;
  char *result;
  
  while (!isalpha (*header))
    header++;

  start = header;

  while (isalnum (*header) || *header == '-' || *header == '/')
    header++;
  result = xmalloc (header - start + 1);
  result[header - start] = '\0';
  memcpy (result, start, header - start);
  return result;
}



static mime_enc_t
default_encoding (mime_type_t type)
{
  switch (type){

    case MTYPE_TEXT_PLAIN:
    case MTYPE_TEXT_OTHER:
    case MTYPE_HTML:
    case MTYPE_MESSAGE:
      return MENC_NONE;

    default:
      return MENC_BASE64;
  }
}



static char *
type_from_type (char *type)
{
  char *result;
  char *seek;

  seek = strstr (type, "text");
  if (seek == type){
    while (isalnum (*seek) || *seek == '/' || *seek == '-')
      seek++;
    result = xmalloc (seek - type + 20 + 1);
    memcpy (result, type, seek - type);
    memcpy (result + (seek - type), "; charset=iso-8859-2", 20);
    result[seek - type + 20] = '\0';
  }
  else {
    result = xstrdup (type);
  }
  return result;
}



static void
make_type (mime_t *mime)
{
  static char buffer[80];
  int   ret;
  int   len     = strlen (mime->file_name);
  char *command = xmalloc (4 + 1 + 2 + 1 + len + 1);
  char *seek;
  FILE *pipe;

  sprintf (command, "file -i %s", mime->file_name);
  pipe = popen (command, "r");
  xfree (command);

  if (pipe == NULL){
    mime->type_desc = NULL;
    mime->type      = MTYPE_UNKNOWN;
    mime->encoding  = MENC_NONE;
    return;
  }

  ret         = fread (buffer, 1, sizeof (buffer) - 1, pipe);
  buffer[ret] = '\0';
  pclose (pipe);

  seek = strchr (buffer, '\n');
  if (seek){
    *seek = '\0';
  }
  
  seek = strrchr (buffer, ':');
  if (seek == NULL)
    seek = buffer;
  do {
    seek++;
  }
  while (!isalnum (*seek));

  mime->type_desc = type_from_type (seek);
  mime->type      = get_type (seek);
  mime->encoding  = default_encoding (mime->type);
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

char *
mime_decode_header (char *s, int free_s)
{
  size_t      len        = strlen (s);
  char       *result     = xmalloc (len + 1);
  char       *result_ptr = result;
  char       *seek       = s;
  char       *start      = s;
  char       *newstart;
  char       *end        = s;
  mime_enc_t  enc;

  while (1){
    enc = find_mime_in_string (start, &seek, &newstart, &end);
    if (seek == NULL || enc == MENC_NONE)
      break;

    memcpy (result_ptr, start, newstart - start);
    result_ptr += newstart - start;
    
    switch (enc){

      case MENC_QP:
        result_ptr = decode_qp (result_ptr, seek, end, 1);
        break;

      case MENC_BASE64:
        result_ptr = decode_base64 (result_ptr, seek, end);
        break;

      default:
        break;
    }
    start = end += 3;
  }

  if (end){
    memcpy (result_ptr, end, len - (end - s) + 1);
  }
  else {
    *result_ptr = '\0';
  }
  if (free_s)
    xfree (s);
  return result;
}



mime_t *
mime_create (mime_type_t type)
{
  mime_t *result = xmalloc (sizeof (mime_t));

  result->type         = type;
  result->offset_start = 0;
  result->offset_end   = 0;
  result->type_desc    = NULL;
  result->file_name    = NULL;
  return result;
}



mime_t *
mime_create_from_filename (char *file_name)
{
  mime_t *result = xmalloc (sizeof (mime_t));

  result->file_name    = xstrdup (file_name);
  result->offset_start = 0;
  result->offset_end   = file_size (file_name);
  make_type (result);
  return result;
}



void
mime_set_from_header (mime_t *mime, char *header)
{
  mime->type      = get_type (header);
  mime->file_name = get_file_name (header);
  if (mime->type != MTYPE_TEXT_PLAIN && mime->type != MTYPE_HTML
      && mime->type != MTYPE_TEXT_OTHER)
    mime->type_desc = get_desc (header);
}



void
mime_complete_file_name (mime_t *mime, char *header)
{
  if (mime->file_name)
    return;
  
  mime->file_name = get_file_name (header);
}



void
mime_destroy (mime_t *mime)
{
  if (mime == NULL)
    return;
  
  if (mime->type_desc)
    xfree (mime->type_desc);
  if (mime->file_name)
    xfree (mime->file_name);
  xfree (mime);
}



char *
mime_decode (mime_t *mime, char *s, size_t *rlen)
{
  char *result = NULL;
  char *end    = NULL;
  int   len    = mime->offset_end - mime->offset_start;
  
  switch (mime->encoding){

    case MENC_NONE:
      result  = xstrdup (s);
      end     = result + len;
      break;

    case MENC_QP:
      result = xmalloc (len + 1);
      end    = decode_qp (result, s, s + len, 0);
      *end   = '\0';
      break;

    case MENC_BASE64:
      result = xmalloc (len + 1);
      end    = decode_base64 (result, s, s + len);
      *end   = '\0';
      break;
  }
  if (rlen)
    *rlen = end - result;
  return result;
}



char *
mime_encode (char *s, size_t len, size_t *rlen, mime_enc_t encoding)
{
  int   slen   = 0;
  char *result = NULL;
  char *end    = NULL;

  switch (encoding){

    case MENC_NONE:
    case MENC_QP:
      result  = xstrdup (s);
      end     = result + len;
      break;

    case MENC_BASE64:
      slen   = (4 * len / 3);
      slen  += slen / 76 + 1;
      slen  += 5;
      result = xmalloc (slen);
      end    = encode_base64 (result, s, s + len - 1);
      *end   = '\0';
      break;
  }
  if (rlen)
    *rlen = end - result;
  return result;
}



char *
mime_get_and_decode (mime_t *mime, const char *file, size_t *rlen)
{
  char *place;
  
  if (file_part (file, mime->offset_start, mime->offset_end, &place)){
    return NULL;
  }
  place = mime_decode (mime, place, rlen);
  return place;
}



char *
mime_get_and_encode (mime_t *mime, size_t *rlen, const char *fname)
{
  const char *file;
        char *place;

  if (mime->file_name == NULL)
    file = fname;
  else
    file = mime->file_name;

  if (file_part (file, mime->offset_start, mime->offset_end, &place)){
    return NULL;
  }
  place = mime_encode (place, mime->offset_end - mime->offset_start, rlen,
                       mime->encoding);

  return place;
}



int
mime_save_in_file (mime_t *mime, const char *sfile, const char *dfile)
{
  char *place;
  FILE *fp;
  int   len;

  fp = fopen (dfile, "w");
  if (!fp)
    return 1;

  place = mime_get_and_decode (mime, sfile, &len);
  fwrite (place, 1, len, fp);
  fclose (fp);
  xfree (place);
  return 0;
}



void
mime_set_content_type (mime_t *mime, char *type)
{
  mime->type_desc = xstrdup (type);
  mime->type      = get_type (type);
  mime->encoding  = default_encoding (mime->type);
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mime.c
 *
 ****************************************************************************/
