/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>

#include "mybox.h"
#include "xmalloc.h"
#include "ask.h"
#include "file.h"
#include "error.h"
#include "read.h"
#include "maildir.h"
#include "gettext.h"
#include "rstring.h"
#include "mail.h"
#include "error.h"
#include "choose.h"
#include "cmd.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static struct dirent **list;
static int             count;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/

char *mybox_dir = NULL;

/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/

static int
select_fun (const struct dirent *entry)
{
  return *entry->d_name != '.';
}


static void
free_list (struct dirent **list, int count)
{
  int i;

  for (i = 0; i < count; i++)
    free (list[i]);

  if (count)
    free (list);

  count = 0;
  list  = NULL;
}


static rstring_t *
get_items (struct dirent **list, int count)
{
  int        i;
  rstring_t *result = rstring_create_size (count + 1);

  for (i = 0; i < count; i++){
    rstring_add (result, list[i]->d_name);
  }

  return result;
}



static char *
box_with_subdir (char *conf, char *def)
{
  char *box;
  
  if (mybox_dir == NULL)
    return NULL;

  box = ask_noninteractive (conf);
  if (box == NULL && def == NULL){
    return NULL;
  }
  else if (box == NULL){
    box = def;
  }

  return file_with_dir (mybox_dir, box);
}


/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

char *
mybox_start (void)
{
  char *box = ask_noninteractive ("mailbox");
  char *box_start;

  if (box != NULL){
    mybox_dir = xstrdup (box);
    return box_with_subdir ("box_start", "inbox");
  }
  
  box_start = getenv ("MAILDIR");
  if (box_start != NULL)
    return xstrdup (box_start);

  box_start = getenv ("MAIL");
  if (box_start != NULL)
    return xstrdup (box_start);

  return file_with_dir ("/var/mail", getlogin ());
}



char *
mybox_sent (void)
{
  return box_with_subdir ("box_sent", "sent");
}



char *
mybox_draft (void)
{
  return box_with_subdir ("box_draft", "drafts");
}



char *
mybox_incoming (void)
{
  return box_with_subdir ("box_incoming", "inbox");
}


char *
mybox_trash (void)
{
  return box_with_subdir ("box_trash", NULL);
}


void
mybox_create_box (void)
{
  char *box = ask_noninteractive ("mailbox");
  char *newdir;

  if (box == NULL){
    error_ (0, 0, _("mailbox not defined"));
    return;
  }

  newdir = read_argument (_("box name: "), NULL);
  if (newdir == NULL || *newdir == '\0')
    return;

  newdir = file_with_dir (box, newdir);
  maildir_create_dir (newdir);
  xfree (newdir);
}



rstring_t *
mybox_subdirs (void)
{
  if (mybox_dir == NULL)
    return NULL;

  count = scandir (mybox_dir, &list, select_fun, alphasort);
  if (count == -1){
    error_ (0, errno, "%s", mybox_dir);
    return NULL;
  }

  return get_items (list, count);
}



void
mybox_free_resources (void)
{
  free_list (list, count);
  if (mybox_dir)
    xfree (mybox_dir);
  mybox_dir = NULL;
}



char *
mybox_select_subdir (void)
{
  char      *answer;
  rstring_t *items;
  int        index;
  
  if (mybox_dir == NULL){
    answer = read_argument ("select box: ", NULL);
    if (answer == NULL || *answer == '\0')
      return NULL;
    return xstrdup (answer);
  }

  items = mybox_subdirs ();
  choose_open (items->count, items->array);

  cmd_choose ();

  index = choose_index ();
  if (index == -1){
    rstring_delete (items);
    return NULL;
  }
  answer = file_with_dir (mybox_dir, items->array[index]);
  rstring_delete (items);

  return answer;
}

/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE mybox.c
 *
 ****************************************************************************/
