#ifndef __POP_H__
#define __POP_H__ 1
/* 
   elmo - Mail User Agent

   Copyright (C) 2002 bazyl, rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/
/*****************************************************************************
 *    INTERFACE REQUIRED HEADERS
 ****************************************************************************/

#include <stdlib.h>

#include "mail.h"

/*****************************************************************************
 *    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/

/**
 * POP3 server sends list of mail in following format:
 *   num size
 * 'num' is index number which should be used when referring to message
 * in retr/dele commands;
 * 'size' is message's size.
 *
 * I use this list to memorize which mails have already been fetched.
 * Every fetched mail is marked as 'fetched'.  Then I store the list
 * of triples (num, size, mark), where mark indicates if mail have
 * alreade been fetched.  Headers of these messages are not fetched
 * from server and these messages are not fetched when using fetch_all.
 */
typedef struct pop_stat {
  int     num;     /* server's number */
  int     size;    /* size in octets */
  int     mark;    /* message have been fetched in one of previous sessions */
  int     fetched; /* message fetched in this session */
  mail_t *info;    /* regular mail_t data */
} pop_stat_t;

/****************************************************************************
 *    INTERFACE DATA DECLARATIONS
 ****************************************************************************/

extern unsigned pop_mail_count;
extern unsigned pop_mail_size;

extern pop_stat_t *pop_stat_list;
extern size_t      pop_stat_size;

/****************************************************************************
 *    INTERFACE FUNCTION PROTOTYPES
 ****************************************************************************/

extern int  pop_begin_connection (char *hostname, unsigned short port);
extern void pop_finish_connection (void);
extern int  pop_authorize (const char *user, const char *pass);
extern int  pop_stat (void);
extern int  pop_list (void);
extern int  pop_list_ext (void);
extern int  pop_dele (int num);
extern int  pop_rset (void);
extern int  pop_retr (int index, FILE *fp);
extern void pop_free_resources (void);

/****************************************************************************
 *    INTERFACE OBJECT CLASS DEFINITIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE TRAILING HEADERS
 ****************************************************************************/
/****************************************************************************
 *
 *    END HEADER pop.h
 *
 ****************************************************************************/
#endif
