/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2002 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   for operating on screen
   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <pthread.h>

#include "screen.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

/* initialized as static variable */
static pthread_mutex_t mutex;

static int finished = 0;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/


int
screen_doupdate (void)
{
  int ret;
  
  if (finished)
    return ERR;
  
  pthread_mutex_lock (&mutex);
  ret = doupdate ();
  pthread_mutex_unlock (&mutex);
  return ret;
}



int
screen_refresh (void)
{
  int ret;
  
  if (finished)
    return ERR;
  
  pthread_mutex_lock (&mutex);
  ret = refresh ();
  pthread_mutex_unlock (&mutex);
  return ret;
}



int
screen_wnoutrefresh (WINDOW *win)
{
  int ret;
  
  if (finished)
    return ERR;
  
  pthread_mutex_lock (&mutex);
  ret = wnoutrefresh (win);
  pthread_mutex_unlock (&mutex);
  return ret;
}



int
screen_wrefresh (WINDOW *win)
{
  int ret;
  
  if (finished)
    return ERR;
  
  pthread_mutex_lock (&mutex);
  ret = wrefresh (win);
  pthread_mutex_unlock (&mutex);
  return ret;
}




int
screen_touchwin (WINDOW *win)
{
  int ret;
  
  if (finished)
    return ERR;
  
  pthread_mutex_lock (&mutex);
  ret = touchwin (win);
  pthread_mutex_unlock (&mutex);
  return ret;
}



int
screen_redrawwin (WINDOW *win)
{
  int ret;

  if (finished)
    return ERR;

  pthread_mutex_lock (&mutex);
  ret = redrawwin (win);
  pthread_mutex_unlock (&mutex);
  return ret;
}



void
screen_free_resources (void)
{
  finished = 1;
  endwin ();
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE screen.c
 *
 ****************************************************************************/
