/* 
   elmo - ELectronic Mail Operator

   Copyright (C) 2003 rzyjontko

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

   ----------------------------------------------------------------------

   
*/
/****************************************************************************
 *    IMPLEMENTATION HEADERS
 ****************************************************************************/

#include <errno.h>

#include "mail.h"
#include "wrapbox.h"
#include "maildir.h"
#include "mbox.h"
#include "mime.h"
#include "rmime.h"
#include "error.h"
#include "mybox.h"
#include "xmalloc.h"

/****************************************************************************
 *    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
 ****************************************************************************/

typedef enum {BOX_INVALID, BOX_MBOX, BOX_MAILDIR} box_type_t;

/****************************************************************************
 *    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE DATA
 ****************************************************************************/

static box_type_t box_type = BOX_INVALID;

/****************************************************************************
 *    INTERFACE DATA
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
 ****************************************************************************/
/****************************************************************************
 *    IMPLEMENTATION PRIVATE FUNCTIONS
 ****************************************************************************/
/****************************************************************************
 *    INTERFACE FUNCTIONS
 ****************************************************************************/

void
wrapbox_free_resources (void)
{
  mbox_free_resources ();
  maildir_free_resources ();
}


int
wrapbox_read_box (char *name)
{
  if (maildir_is_valid (name)){
    box_type = BOX_MAILDIR;
    return maildir_read_dir (name);
  }

  if (mbox_may_be_valid (name)){
    box_type = BOX_MBOX;
    return mbox_read_file (name);
  }

  box_type = BOX_INVALID;
  return -1;
}



int
wrapbox_mail_header (mail_t *mail, char **place)
{
  int ret = 0;

  switch (box_type){

    case BOX_INVALID:
      return 1;
    
    case BOX_MBOX:
      ret = mbox_mail_header (mail, place);
      break;

    case BOX_MAILDIR:
      ret = maildir_mail_header (mail, place);
      break;
  }

  *place = mime_decode_header (*place, 0);

  return ret;
}



int
wrapbox_mail_body (mail_t *mail, char **place, int mime_index)
{
  int     ret = 0;
  mime_t *mime;
  
  if (mime_index < 0 || mime_index >= mail->rmime->count){
    mime = rmime_first_text (mail->rmime);
    if (mime == NULL){
      errno = 0;
      return 1;
    }
  }
  else {
    mime = mail->rmime->array[mime_index];
  }

  switch (box_type){

    case BOX_INVALID:
      error_ (0, 0, "this IS a BUG, please submit");
      return 1;
    
    case BOX_MBOX:
      ret = mbox_mail_body (mail, place, mime);
      break;

    case BOX_MAILDIR:
      ret = maildir_mail_body (mail, place, mime);
      break;
  }

  return ret;
}




void
wrapbox_add_flag (mail_t *mail, flag_t flag)
{
  switch (box_type){

    case BOX_INVALID:
      break;

    case BOX_MBOX:
      break;

    case BOX_MAILDIR:
      maildir_add_flag (mail, flag);
      break;
  }
}



void
wrapbox_remove_flag (mail_t *mail, flag_t flag)
{
  switch (box_type){

    case BOX_INVALID:
      break;

    case BOX_MBOX:
      break;

    case BOX_MAILDIR:
      maildir_remove_flag (mail, flag);
      break;
  }
}



int
wrapbox_move_mail_to (mail_t *mail, char *box)
{
  int ret;
  
  if (mail->flags & FLAG_MAILDIR){
    ret = maildir_move_mail_to (mail, box);
    return ret;
  }
  return 1;
}



void
wrapbox_remove_trashed (void)
{
  int   i;
  char *trash = mybox_trash ();

  if (trash){
    for (i = 0; i < mail_count; i++){
      if (mail_array[i].flags & FLAG_TRASHED){
        if (wrapbox_move_mail_to (mail_array + i, trash))
          return;
      }
    }
    xfree (trash);
    return;
  }
  
  switch (box_type){

    case BOX_INVALID:
      break;

    case BOX_MBOX:
      break;

    case BOX_MAILDIR:
      maildir_remove_trashed ();
      break;
  }
}



int
wrapbox_mail_size (mail_t *mail)
{
  switch (box_type){

    case BOX_INVALID:
      return 0;

    case BOX_MBOX:
      return mbox_mail_size (mail);

    case BOX_MAILDIR:
      return maildir_mail_size (mail);

    default:
      return 0;
  }
}



int
wrapbox_mail_count (const char *box, int *unread)
{
  if (maildir_is_valid (box)){
    return maildir_mail_count (box, unread);
  }

  if (unread)
    *unread = 0;
  return 0;
}


/****************************************************************************
 *    INTERFACE CLASS BODIES
 ****************************************************************************/
/****************************************************************************
 *
 *    END MODULE wrapbox.c
 *
 ****************************************************************************/
